﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS {
  [Item("MatingPoolCreator", "An operator which creating mating pools based on a set of sub-populations. For each sub-population, the individuals from the previous sub-population are copied into the current sub-population.")]
  [StorableClass]
  public sealed class MatingPoolCreator : SingleSuccessorOperator {

    public ILookupParameter<IntValue> MatingPoolRangeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["MatingPoolRange"]; }
    }
    public ILookupParameter<PercentValue> MatingPoolSelectionPercentageParameter {
      get { return (ILookupParameter<PercentValue>)Parameters["MatingPoolSelectionPercentage"]; }
    }

    [StorableConstructor]
    private MatingPoolCreator(bool deserializing)
      : base(deserializing) { }
    private MatingPoolCreator(MatingPoolCreator original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MatingPoolCreator(this, cloner);
    }
    public MatingPoolCreator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("MatingPoolRange"));
      Parameters.Add(new LookupParameter<PercentValue>("MatingPoolSelectionPercentage"));
    }

    public override IOperation Apply() {
      var layers = ExecutionContext.Scope.SubScopes;
      int range = MatingPoolRangeParameter.ActualValue.Value;
      double percentage = MatingPoolSelectionPercentageParameter.ActualValue.Value;

      for (int layer = layers.Count - 1; layer > 0; layer--) {
        var layerScope = layers[layer];
        for (int n = 1; (n <= range) && (layer - n >= 0); n++) {
          var prevLayerScope = layers[layer - n];
          var individuals = prevLayerScope.SubScopes;
          foreach (var individual in individuals.Take((int)(individuals.Count * percentage))) {
            layerScope.SubScopes.Add((IScope)individual.Clone(new Cloner()));
          }
        }
      }

      return base.Apply();
    }
  }
}